/*
Module : EventLogClient.h
Purpose: Defines the interface for a set of C++ wrapper classes for the client side (i.e. reading) from 
         the Windows Event log available in Windows Vista and later
Created: PJN / 19-06-2016
History: PJN / 19-06-2016 1. Initial public release
         PJN / 25-11-2018 1. Updated copyright details
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean 
                          that the code will now only compile on VC 2017 or later.
         PJN / 21-04-2019 1. Updated copyright details
                          2. Updated the code to clean compile on VC 2019
         PJN / 03-10-2019 1. Fixed a number of compiler warnings when the code is compiled with VS 2019 Preview
         PJN / 18-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 29-01-2022 1. Updated copyright details.
                          2. Fixed more static code analysis warnings in Visual Studio 2022.
         PJN / 17-03-2022 1. Updated the code to use C++ uniform initialization for all variable declarations.
                          2. Updated the code to use std::vector::data method throughout. This means that the 
                          code must now be compiled using /std:c++17.
         PJN / 15-05-2023 1. Updated copyright details
                          2. Updated modules to indicate that it needs to be compiled using /std:c++17. Thanks
                          to Martin Richter for reporting this issue.

Copyright (c) 2016 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


////////////////////////// Macros / Defines ///////////////////////////////////

#pragma once

#if _MSVC_LANG < 201703
#error EventLogWrappers requires a minimum C++ language standard of /std:c++17
#endif //#if _MSVC_LANG < 201703

#ifndef __EVENTLOGCLIENT_H__
#define __EVENTLOGCLIENT_H__

#pragma comment(lib, "wevtapi.lib")


///////////////////////// Includes ////////////////////////////////////////////

#ifndef __WINEVT_H__
#pragma message("To avoid this message, please put winevt.h in your pre compiled header (normally stdafx.h)")
#include <winevt.h>
#endif //#ifndef __WINEVT_H__

#ifndef _STRING_
#pragma message("To avoid this message, please put string in your pre compiled header (normally stdafx.h)")
#include <string>
#endif //#ifndef _STRING_

#ifndef _VECTOR_
#pragma message("To avoid this message, please put vector in your pre compiled header (normally stdafx.h)")
#include <vector>
#endif //#ifndef _VECTOR_

#ifndef _UTILITY_
#pragma message("To avoid this message, please put utility in your pre compiled header (normally stdafx.h)")
#include <utility>
#endif //#ifndef _UTILITY_

///////////////////////// Classes /////////////////////////////////////////////

namespace EventLog
{
//Functions
inline BOOL GetExtendedStatus(_Inout_ std::wstring& sValue)
{
  //What will be the return value from this function (assume the worst)
  BOOL bSuccess{FALSE};

  //Set the output parameter to a default value
  sValue.clear();

  DWORD dwBuffer{0};
  if (EvtGetExtendedStatus(0, nullptr, &dwBuffer) == ERROR_INSUFFICIENT_BUFFER)
  {
    sValue.resize(dwBuffer);
#pragma warning(suppress: 26446 26489)
    bSuccess = (EvtGetExtendedStatus(dwBuffer, &(sValue[0]), &dwBuffer) == ERROR_SUCCESS);
    if (bSuccess)
#pragma warning(suppress: 26489)
      sValue.resize(dwBuffer);
  }

  return bSuccess;
}


//Client side functionality


//Class wrapper for a Event logging handle (EVT_HANDLE)
class CHandle
{
public:
//Constructors / Destructors
  CHandle() noexcept : m_h{nullptr}
  {
  }

  CHandle(const CHandle&) = delete;

  CHandle(_In_ CHandle&& h) noexcept : m_h{h.m_h}
  {
    h.m_h = nullptr;
  }

  CHandle(_In_ EVT_HANDLE h) noexcept : m_h{h}
  {
  }

  virtual ~CHandle() noexcept
  {
    if (m_h != nullptr)
      Close();
  }

  CHandle& operator=(const CHandle&) = delete;

  CHandle& operator=(_In_ CHandle&& h) noexcept
  {
    if (m_h != nullptr)
      Close();
    Attach(h.Detach());
    return *this;
  }

//Methods
  operator EVT_HANDLE() const noexcept
  {
    return m_h;
  }

  void Attach(_In_ EVT_HANDLE h) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSUME(m_h == nullptr);

    m_h = h;
  }

  EVT_HANDLE Detach() noexcept
  {
    EVT_HANDLE h{m_h};
    m_h = nullptr;
    return h;
  }

  BOOL Close() noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    const BOOL bSuccess{EvtClose(m_h)};
    m_h = nullptr;
    return bSuccess;
  }

  BOOL Cancel() noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtCancel(m_h);
  }

//Member variables
  EVT_HANDLE m_h;
};

//Class wrapper for the handle(s) returned from EvtNext
class CEvent : public CHandle
{
public:
//Constructors / Destructors
  CEvent() = default;

  CEvent(const CEvent&) = delete;

  CEvent(_In_ CEvent&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CEvent(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CEvent() = default; //NOLINT(modernize-use-override)

//Methods
  CEvent& operator=(const CEvent&) = delete;

#pragma warning(suppress: 26456)
  CEvent& operator=(_In_ CEvent&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

#pragma warning(suppress: 26812)
  BOOL GetInfo(_In_ EVT_EVENT_PROPERTY_ID PropertyId, _In_ DWORD PropertyValueBufferSize, _In_opt_ PEVT_VARIANT PropertyValueBuffer, _Out_ PDWORD PropertyValueBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtGetEventInfo(m_h, PropertyId, PropertyValueBufferSize, PropertyValueBuffer, PropertyValueBufferUsed);
  }
};

//Class wrapper for the handle returned from bookmarks
class CBookmark : public CHandle
{
public:
//Constructors / Destructors
  CBookmark() = default;

  CBookmark(const CBookmark&) = delete;

  CBookmark(_In_ CBookmark&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CBookmark(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CBookmark() = default; //NOLINT(modernize-use-override)

//Methods
  CBookmark& operator=(const CBookmark&) = delete;

#pragma warning(suppress: 26456)
  CBookmark& operator=(_In_ CBookmark&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  BOOL Create(_In_opt_ LPCWSTR BookmarkXml) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h == nullptr);

    m_h = EvtCreateBookmark(BookmarkXml);
    return (m_h != nullptr);
  }

  BOOL Update(_In_ const CEvent& event) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtUpdateBookmark(m_h, event);
  }
};

//Class wrapper for the handle returned from EvtQuery or EvtSubscribe
class CResultSet : public CHandle
{
public:
//Constructors / Destructors
  CResultSet() = default;

  CResultSet(const CResultSet&) = delete;

  CResultSet(_In_ CResultSet&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CResultSet(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CResultSet() = default; //NOLINT(modernize-use-override)

//Methods
  CResultSet& operator=(const CResultSet&) = delete;

#pragma warning(suppress: 26456)
  CResultSet& operator=(_In_ CResultSet&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  BOOL Next(_In_ DWORD EventsSize, _In_ PEVT_HANDLE Events, _In_ DWORD Timeout, _In_ DWORD Flags, _Out_ PDWORD Returned) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtNext(m_h, EventsSize, Events, Timeout, Flags, Returned);
  }

  BOOL Next(_Inout_ std::vector<CEvent>& Events, _In_ DWORD Timeout, _In_ DWORD Flags, _Out_ PDWORD Returned)
  {
#pragma warning(suppress: 26472)
    const auto dwHandles{static_cast<DWORD>(Events.size())};
    std::vector<EVT_HANDLE> handles{dwHandles, std::allocator<EVT_HANDLE>{}};
#pragma warning(suppress: 26446)
    const BOOL bSuccess{Next(dwHandles, handles.data(), Timeout, Flags, Returned)};
    if (bSuccess)
    {
        for (DWORD i{0}; i<dwHandles; i++)
        {
#pragma warning(suppress: 26446)
          CEvent& evt{Events[i]};
          if (evt.operator EVT_HANDLE())
            evt.Close();
#pragma warning(suppress: 26446)
          evt.Attach(handles[i]);
        }
    }
    return bSuccess;
  }

  BOOL Seek(_In_ LONGLONG Position, _In_opt_ EVT_HANDLE Bookmark, _Reserved_ DWORD Timeout, _In_ DWORD Flags) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtSeek(m_h, Position, Bookmark, Timeout, Flags);
  }

#pragma warning(suppress: 26812)
  BOOL GetInfo(EVT_QUERY_PROPERTY_ID PropertyId, DWORD PropertyValueBufferSize, PEVT_VARIANT PropertyValueBuffer, _Out_ PDWORD PropertyValueBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtGetQueryInfo(m_h, PropertyId, PropertyValueBufferSize, PropertyValueBuffer, PropertyValueBufferUsed);
  }
};

//Class wrapper for the handle returned from EvtOpenLog
class CLog : public CHandle
{
public:
//Constructors / Destructors
  CLog() = default;

  CLog(const CLog&) = delete;

  CLog(_In_ CLog&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CLog(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CLog() = default; //NOLINT(modernize-use-override)

//Methods
  CLog& operator=(const CLog&) = delete;

#pragma warning(suppress: 26456)
  CLog& operator=(_In_ CLog&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

#pragma warning(suppress: 26812)
  BOOL GetInfo(_In_ EVT_LOG_PROPERTY_ID PropertyId, _In_ DWORD PropertyValueBufferSize, _In_opt_ PEVT_VARIANT PropertyValueBuffer, _Out_ PDWORD PropertyValueBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtGetLogInfo(m_h, PropertyId, PropertyValueBufferSize, PropertyValueBuffer, PropertyValueBufferUsed);
  }
};

//Class wrapper for the handle returned from EvtOpenChannelEnum
class CChannelEnumerator : public CHandle
{
public:
//Constructors / Destructors
  CChannelEnumerator() = default;

  CChannelEnumerator(const CChannelEnumerator&) = delete;

  CChannelEnumerator(_In_ CChannelEnumerator&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CChannelEnumerator(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CChannelEnumerator() = default; //NOLINT(modernize-use-override)

//Methods
  CChannelEnumerator& operator=(const CChannelEnumerator&) = delete;

#pragma warning(suppress: 26456)
  CChannelEnumerator& operator=(_In_ CChannelEnumerator&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  BOOL Next(_In_ DWORD ChannelPathBufferSize, _Out_writes_to_opt_(ChannelPathBufferSize, *ChannelPathBufferUsed) LPWSTR ChannelPathBuffer, _Out_ PDWORD ChannelPathBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtNextChannelPath(m_h, ChannelPathBufferSize, ChannelPathBuffer, ChannelPathBufferUsed);
  }

  BOOL Next(_Inout_ std::wstring& sChannelPath)
  {
    //What will be the return value from this method (assume the worst)
    BOOL bSuccess{FALSE};

    sChannelPath.clear();

    DWORD dwBufferSize{0};
    if (!Next(dwBufferSize, nullptr, &dwBufferSize))
    {
      const DWORD dwError{GetLastError()};
      if (dwError == ERROR_INSUFFICIENT_BUFFER)
      {
        sChannelPath.resize(dwBufferSize);
#pragma warning(suppress: 26446 26489)
        bSuccess = Next(dwBufferSize, sChannelPath.data(), &dwBufferSize);
        if (bSuccess)
#pragma warning(suppress: 26489)
          sChannelPath.resize(dwBufferSize);
      }
    }

    return bSuccess;
  }
};

//Class wrapper for the handle returned from EvtOpenChannelConfig
class CChannelConfig : public CHandle
{
public:
//Constructors / Destructors
  CChannelConfig() = default;

  CChannelConfig(const CChannelConfig&) = delete;

  CChannelConfig(_In_ CChannelConfig&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CChannelConfig(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CChannelConfig() = default; //NOLINT(modernize-use-override)

//Methods
  CChannelConfig& operator=(const CChannelConfig&) = delete;

#pragma warning(suppress: 26456)
  CChannelConfig& operator=(_In_ CChannelConfig&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  BOOL Save(_In_ DWORD Flags = 0) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtSaveChannelConfig(m_h, Flags);
  }

#pragma warning(suppress: 26812)
  BOOL SetProperty(_In_ EVT_CHANNEL_CONFIG_PROPERTY_ID PropertyId, _In_ DWORD Flags, _In_ PEVT_VARIANT PropertyValue) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtSetChannelConfigProperty(m_h, PropertyId, Flags, PropertyValue);
  }

  BOOL GetProperty(_In_ EVT_CHANNEL_CONFIG_PROPERTY_ID PropertyId, _In_ DWORD Flags, _In_ DWORD PropertyValueBufferSize, _In_opt_ PEVT_VARIANT PropertyValueBuffer, _Out_ PDWORD PropertyValueBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtGetChannelConfigProperty(m_h, PropertyId, Flags, PropertyValueBufferSize, PropertyValueBuffer, PropertyValueBufferUsed);
  }
};

//Class wrapper for the handle returned from EvtNextEventMetadata
class CEventMetaData : public CHandle
{
public:
  //Constructors / Destructors
  CEventMetaData() = default;

  CEventMetaData(const CEventMetaData&) = delete;

  CEventMetaData(_In_ CEventMetaData&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CEventMetaData(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CEventMetaData() = default; //NOLINT(modernize-use-override)

//Methods
  CEventMetaData& operator=(const CEventMetaData&) = delete;

#pragma warning(suppress: 26456)
  CEventMetaData& operator=(_In_ CEventMetaData&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

#pragma warning(suppress: 26812)
  BOOL GetProperty(_In_ EVT_EVENT_METADATA_PROPERTY_ID PropertyId, _In_ DWORD Flags, _In_ DWORD EventMetadataPropertyBufferSize,
                   _In_opt_ PEVT_VARIANT EventMetadataPropertyBuffer, _Out_ PDWORD EventMetadataPropertyBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtGetEventMetadataProperty(m_h, PropertyId, Flags, EventMetadataPropertyBufferSize, EventMetadataPropertyBuffer, EventMetadataPropertyBufferUsed);
  }
};

//Class wrapper for the handle returned from EvtOpenEventMetadataEnum
class CEventEnumerator : public CHandle
{
public:
//Constructors / Destructors
  CEventEnumerator() = default;

  CEventEnumerator(const CEventEnumerator&) = delete;

  CEventEnumerator(_In_ CEventEnumerator&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CEventEnumerator(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CEventEnumerator() = default; //NOLINT(modernize-use-override)

//Methods
  CEventEnumerator& operator=(const CEventEnumerator&) = delete;

#pragma warning(suppress: 26456)
  CEventEnumerator& operator=(_In_ CEventEnumerator&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  _NODISCARD CEventMetaData Next(_Reserved_ DWORD Flags = 0) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return {EvtNextEventMetadata(m_h, Flags)};
  }
};

//Class wrapper for the handle returned from EvtOpenPublisherMetadata
class CPublisher : public CHandle
{
public:
//Constructors / Destructors
  CPublisher() = default;

  CPublisher(const CPublisher&) = delete;

  CPublisher(_In_ CPublisher&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CPublisher(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CPublisher() = default; //NOLINT(modernize-use-override)

//Methods
  CPublisher& operator=(_In_ const CPublisher& h) = delete;

#pragma warning(suppress: 26456)
  CPublisher& operator=(_In_ CPublisher&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  BOOL FormatMessage(_In_ const CEvent& evt, _In_ DWORD MessageId, _In_ DWORD ValueCount, _In_opt_ PEVT_VARIANT Values, _In_ DWORD Flags,
                     _In_ DWORD BufferSize, _Out_writes_to_opt_(BufferSize, *BufferUsed) LPWSTR Buffer, _Out_ PDWORD BufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

#pragma warning(suppress: 6387) //EvtFormatMessage even in the latest Windows 10 SDK has incorrect SAL annotations for "Values"
    return EvtFormatMessage(m_h, evt, MessageId, ValueCount, Values, Flags, BufferSize, Buffer, BufferUsed);
  }

  BOOL FormatMessage(_In_ const CEvent& evt, _In_ DWORD MessageId, _In_ DWORD ValueCount, _In_opt_ PEVT_VARIANT Values, _In_ DWORD Flags, _Inout_ std::wstring& sValue)
  {
    //What will be the return value from this method (assume the worst)
    BOOL bSuccess{FALSE};

    sValue.clear();

    DWORD dwBufferSize{0};
    if (!FormatMessage(evt, MessageId, ValueCount, Values, Flags, 0, nullptr, &dwBufferSize))
    {
      const DWORD dwError{GetLastError()};
      if (dwError == ERROR_INSUFFICIENT_BUFFER)
      {
        sValue.resize(dwBufferSize);
#pragma warning(suppress: 26446 26489)
        bSuccess = FormatMessage(evt, MessageId, ValueCount, Values, Flags, dwBufferSize, sValue.data(), & dwBufferSize);
        if (bSuccess)
#pragma warning(suppress: 26489)
          sValue.resize(dwBufferSize);
      }
    }

    return bSuccess;
  }

#pragma warning(suppress: 26812)
  BOOL GetProperty(_In_ EVT_PUBLISHER_METADATA_PROPERTY_ID PropertyId, _In_ DWORD Flags, _In_ DWORD PublisherMetadataPropertyBufferSize,
                   _In_opt_ PEVT_VARIANT PublisherMetadataPropertyBuffer, _Out_ PDWORD PublisherMetadataPropertyBufferUsed) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return EvtGetPublisherMetadataProperty(m_h, PropertyId, Flags, PublisherMetadataPropertyBufferSize,PublisherMetadataPropertyBuffer, PublisherMetadataPropertyBufferUsed);
  }

  _NODISCARD CEventEnumerator EnumEvents(_In_ DWORD Flags = 0) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != nullptr);

    return {EvtOpenEventMetadataEnum(m_h, Flags)};
  }
};

//Class wrapper for the handle returned from EvtOpenPublisherEnum
class CPublisherEnumerator : public CHandle
{
public:
//Constructors / Destructors
  CPublisherEnumerator() = default;

  CPublisherEnumerator(const CPublisherEnumerator&) = delete;

  CPublisherEnumerator(_In_ CPublisherEnumerator&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CPublisherEnumerator(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CPublisherEnumerator() = default; //NOLINT(modernize-use-override)

//Methods
  CPublisherEnumerator& operator=(_In_ const CPublisherEnumerator& h) = delete;

#pragma warning(suppress: 26456)
  CPublisherEnumerator& operator=(_In_ CPublisherEnumerator&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

  BOOL Next(_In_ DWORD PublisherIdBufferSize, _Out_writes_to_opt_(PublisherIdBufferSize, *PublisherIdBufferUsed) LPWSTR PublisherIdBuffer, _Out_ PDWORD PublisherIdBufferUsed) noexcept
  {
    return EvtNextPublisherId(m_h, PublisherIdBufferSize, PublisherIdBuffer, PublisherIdBufferUsed);
  }

  BOOL Next(_Inout_ std::wstring& sProvider)
  {
    //What will be the return value from this method (assume the worst)
    BOOL bSuccess{FALSE};

    sProvider.clear();

    DWORD dwBufferSize{0};
    if (!Next(dwBufferSize, nullptr, &dwBufferSize))
    {
      const DWORD dwError{GetLastError()};
      if (dwError == ERROR_INSUFFICIENT_BUFFER)
      {
        sProvider.resize(dwBufferSize);
#pragma warning(suppress: 26446 26489)
        bSuccess = Next(dwBufferSize, sProvider.data(), &dwBufferSize);
        if (bSuccess)
#pragma warning(suppress: 26489)
          sProvider.resize(dwBufferSize);
      }
    }

    return bSuccess;
  }
};

//Class wrapper for the handle returned from EvtOpenSession or the implicit session handle for the local machine
class CSession : public CHandle
{
public:
//Constructors / Destructors
  CSession() = default;

  CSession(const CSession&) = delete;

  CSession(_In_ CSession&& h) noexcept : CHandle{std::move(h)}
  {
  }

  CSession(_In_ EVT_HANDLE h) noexcept : CHandle{h}
  {
  }

  ~CSession() = default; //NOLINT(modernize-use-override)

//Methods
  CSession& operator=(_In_ const CSession& h) = delete;

#pragma warning(suppress: 26456)
  CSession& operator=(_In_ CSession&& h) noexcept
  {
    Attach(h.Detach());
    return *this;
  }

#pragma warning(suppress: 26812)
  BOOL Open(_In_ EVT_LOGIN_CLASS LoginClass, _In_ PVOID Login, _Reserved_ DWORD Timeout = 0, _Reserved_ DWORD Flags = 0) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h == nullptr);

    m_h = EvtOpenSession(LoginClass, Login, Timeout, Flags);
    return (m_h != nullptr);
  }

  _NODISCARD CResultSet Query(_In_ LPCWSTR Path, _In_opt_ LPCWSTR Query, _In_ DWORD Flags) noexcept
  {
    return {EvtQuery(m_h, Path, Query, Flags)};
  }

  _NODISCARD CResultSet Subscribe(_In_opt_ HANDLE SignalEvent, _In_ LPCWSTR ChannelPath, _In_opt_ LPCWSTR Query, 
                                  _In_opt_ EVT_HANDLE Bookmark, _In_opt_ PVOID context, _In_opt_ EVT_SUBSCRIBE_CALLBACK Callback, _In_ DWORD Flags) noexcept
  {
    return {EvtSubscribe(m_h, SignalEvent, ChannelPath, Query, Bookmark, context, Callback, Flags)};
  }

  BOOL ClearLog(_In_ LPCWSTR ChannelPath, _In_opt_ LPCWSTR TargetFilePath, _In_ DWORD Flags = 0) noexcept
  {
    return EvtClearLog(m_h, ChannelPath, TargetFilePath, Flags);
  }

  _NODISCARD CLog OpenLog(_In_ LPCWSTR Path, _In_ DWORD Flags) noexcept
  {
    return {EvtOpenLog(m_h, Path, Flags)};
  }

  BOOL ExportLog(_In_opt_ LPCWSTR Path, _In_opt_ LPCWSTR Query, _In_ LPCWSTR TargetFilePath, _In_ DWORD Flags) noexcept
  {
    return EvtExportLog(m_h, Path, Query, TargetFilePath, Flags);
  }

  BOOL ArchiveExportedLog(_In_ LPCWSTR LogFilePath, _In_ LCID Locale, _In_ DWORD Flags = 0) noexcept
  {
    return EvtArchiveExportedLog(m_h, LogFilePath, Locale, Flags);
  }

  _NODISCARD CChannelEnumerator OpenChannelEnum(_In_ DWORD Flags = 0) noexcept
  {
    return {EvtOpenChannelEnum(m_h, Flags)};
  }

  _NODISCARD CChannelConfig OpenChannelConfig(_In_ LPCWSTR ChannelPath, _In_ DWORD Flags = 0) noexcept
  {
    return {EvtOpenChannelConfig(m_h, ChannelPath, Flags)};
  }

  _NODISCARD CPublisher OpenPublisher(_In_z_ LPCWSTR PublisherIdentity, _In_opt_z_ LPCWSTR LogFilePath, _In_ LCID Locale = 0, _In_ DWORD Flags = 0) noexcept
  {
    return {EvtOpenPublisherMetadata(m_h, PublisherIdentity, LogFilePath, Locale, Flags)};
  }

  _NODISCARD CPublisherEnumerator OpenPublisherEnum(_In_ DWORD Flags = 0) noexcept
  {
    return {EvtOpenPublisherEnum(m_h, Flags)};
  }
};


}; //namespace EventLog

#endif //#ifndef __EVENTLOGCLIENT_H__
