/*
Module : EventLogProvider.h
Purpose: Defines the interface for a set of C++ wrapper classes for the provider side (i.e. writing) to 
         the Windows Event log available in Windows Vista and later
Created: PJN / 19-06-2016
History: PJN / 19-06-2016 1. Initial public release
         PJN / 25-11-2018 1. Updated copyright details
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean
                          that the code will now only compile on VC 2017 or later.
         PJN / 21-04-2019 1. Updated copyright details
                          2. Updated the code to clean compile on VC 2019
         PJN / 03-10-2019 1. Fixed a number of compiler warnings when the code is compiled with VS 2019 Preview
         PJN / 18-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 29-01-2022 1. Updated copyright details.
                          2. Fixed more static code analysis warnings in Visual Studio 2022.
         PJN / 17-03-2022 1. Updated the code to use C++ uniform initialization for all variable declarations.
         PJN / 15-05-2023 1. Updated copyright details
                          2. Updated modules to indicate that it needs to be compiled using /std:c++17. Thanks 
                          to Martin Richter for reporting this issue.

Copyright (c) 2016 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


////////////////////////// Macros / Defines ///////////////////////////////////

#pragma once

#if _MSVC_LANG < 201703
#error EventLogWrappers requires a minimum C++ language standard of /std:c++17
#endif //#if _MSVC_LANG < 201703

#ifndef __EVENTLOGPROVIDER_H__
#define __EVENTLOGPROVIDER_H__

#pragma comment(lib, "AdvAPI32.Lib")


///////////////////////// Includes ////////////////////////////////////////////

#ifndef _EVNTPROV_H_
#pragma message("To avoid this message, please put evntprov.h in your pre compiled header (normally stdafx.h)")
#include <evntprov.h>
#endif //#ifndef _EVNTPROV_H_


///////////////////////// Classes /////////////////////////////////////////////

namespace EventLog
{

//Class wrapper for the handle as returned from EventRegister
class CProvider
{
public:
//Constructors / Destructors
  CProvider() noexcept : m_h{0}
  {
  }

  CProvider(const CProvider&) = delete;

  CProvider(_In_ CProvider&& h) noexcept : m_h{std::move(h.m_h)}
  {
  }

  CProvider(_In_ REGHANDLE h) noexcept : m_h{h}
  {
  }

  virtual ~CProvider() noexcept
  {
    if (m_h != 0)
      Unregister();
  }

  CProvider& operator=(const CProvider&) = delete;

  CProvider& operator=(_In_ CProvider&& h) noexcept
  {
    if (m_h != 0)
      Unregister();
    Attach(h.Detach());
    return *this;
  }

//Methods
  operator REGHANDLE() const noexcept
  {
    return m_h;
  }

  void Attach(_In_ REGHANDLE h) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSUME(m_h == 0);

    m_h = h;
  }

  _NODISCARD REGHANDLE Detach() noexcept
  {
    const REGHANDLE h{m_h};
    m_h = 0;
    return h;
  }

  ULONG Register(_In_ LPCGUID ProviderId, _In_opt_ PENABLECALLBACK EnableCallback = nullptr, _In_opt_ PVOID CallbackContext = nullptr) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h == 0);

    return EventRegister(ProviderId, EnableCallback, CallbackContext, &m_h);
  }

  ULONG Unregister() noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    const ULONG nReturn{EventUnregister(m_h)};
    m_h = 0;
    return nReturn;
  }

__if_exists(EventSetInformation)
{
#pragma warning(suppress: 26812)
  ULONG SetInformation(_In_ EVENT_INFO_CLASS InformationClass, _In_reads_bytes_(InformationLength) PVOID EventInformation, _In_ ULONG InformationLength) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return EventSetInformation(m_h, InformationClass, EventInformation, InformationLength);
  }
} //__if_exists(EventSetInformation)

  BOOLEAN EventEnabled(_In_ PCEVENT_DESCRIPTOR EventDescriptor) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return ::EventEnabled(m_h, EventDescriptor);
  }

  BOOLEAN Enabled(_In_ UCHAR Level, _In_ ULONGLONG Keyword) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return EventProviderEnabled(m_h, Level, Keyword);
  }

  ULONG Write(_In_ PCEVENT_DESCRIPTOR EventDescriptor, _In_ ULONG UserDataCount, _In_reads_opt_(UserDataCount) PEVENT_DATA_DESCRIPTOR UserData) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return EventWrite(m_h, EventDescriptor, UserDataCount, UserData);
  }

  ULONG WriteTransfer(_In_ PCEVENT_DESCRIPTOR EventDescriptor, _In_opt_ LPCGUID ActivityId, _In_opt_ LPCGUID RelatedActivityId,
                      _In_ ULONG UserDataCount, _In_reads_opt_(UserDataCount) PEVENT_DATA_DESCRIPTOR UserData) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return EventWriteTransfer(m_h, EventDescriptor, ActivityId, RelatedActivityId, UserDataCount, UserData);
  }

__if_exists(EventWriteEx)
{
  ULONG Write(_In_ PCEVENT_DESCRIPTOR EventDescriptor, _In_ ULONG64 Filter, _In_ ULONG Flags, _In_opt_ LPCGUID ActivityId,
              _In_opt_ LPCGUID RelatedActivityId, _In_ ULONG UserDataCount, _In_reads_opt_(UserDataCount) PEVENT_DATA_DESCRIPTOR UserData) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return EventWriteEx(m_h, EventDescriptor, Filter, Flags, ActivityId, RelatedActivityId, UserDataCount,UserData);
  }
} //__if_exists(EventWriteEx)

  ULONG Write(_In_ UCHAR Level, _In_ ULONGLONG Keyword, _In_ PCWSTR String) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_h != 0);

    return EventWriteString(m_h, Level, Keyword, String);
  }

//Member variables
  REGHANDLE m_h;
};


}; //namespace EventLog

#endif //#ifndef __EVENTLOGPROVIDER_H__
